@@ -2,7 +2,7 @@
 #
 # Author: Yipeng Sun
 # License: BSD 2-clause
-# Last Change: Wed Jun 24, 2020 at 09:24 PM +0800
+# Last Change: Mon May 24, 2021 at 10:06 PM +0200
 
 import uproot
 
@@ -37,15 +37,17 @@ class CutflowRule:
     key: Optional[str] = None
 
 
-class CutflowGen(object):
+class CutflowGen:
     def __init__(self, ntp_path, tree, rules, init_num, **kwargs):
-        self.rules = rules
+        self.rules = self.strip_multiline_str(rules)
         self.init_num = init_num
 
         self.ntp = uproot.open(ntp_path)
         self.tree = tree
         self.exe = BooleanEvaluator(self.ntp, tree, **kwargs)
 
+        self.debug_raw_output = []
+
     def do(self, output_regulator=lambda ntp, tree, arr: sum(arr)):
         ref = {}
         result = {}
@@ -66,9 +68,14 @@ class CutflowGen(object):
             raw_output = self.exe.eval(r.cond)
             if not r.explicit:
                 raw_output = AND(prev_raw_output, raw_output)
+            self.debug_raw_output.append(raw_output)
 
             # Here, 'output' is a number
-            output = output_regulator(self.ntp, self.tree, raw_output)
+            if True in raw_output:
+                output = output_regulator(self.ntp, self.tree, raw_output)
+            else:
+                output = 0
+
             cut_result = {'input': prev_output, 'output': output}
 
             if r.name:
@@ -95,3 +102,14 @@ class CutflowGen(object):
             idx = int(raw_idx)
 
         return idx
+
+    @staticmethod
+    def strip_multiline_str(rules):
+        result = []
+        for r in rules:
+            stripped_rule = CutflowRule(
+                r.cond.replace('\n', ' '),
+                r.name, r.compare_to, r.explicit, r.key
+            )
+            result.append(stripped_rule)
+        return result
