@@ -2,7 +2,7 @@
 #
 # Author: Yipeng Sun
 # License: BSD 2-clause
-# Last Change: Mon May 31, 2021 at 03:10 AM +0200
+# Last Change: Mon Jun 14, 2021 at 06:04 PM +0200
 
 import numpy as np
 import matplotlib as mp
@@ -34,13 +34,13 @@ def decorate_output(f, tight_layout=True, pad=0.):
 
         if not output:
             return data
-        else:
-            fig = data[0]
-            if tight_layout:
-                plt.tight_layout(pad=pad)
-            fig.savefig(output)
-            fig.clf()
-            plt.close(fig)
+
+        fig = data[0]
+        if tight_layout:
+            plt.tight_layout(pad=pad)
+        fig.savefig(output)
+        fig.clf()
+        plt.close(fig)
 
     return wrapper
 
@@ -62,73 +62,20 @@ def decorate_ax_style(f, *args, **kwargs):
 # Plot helpers #
 ################
 
-def plot_style(text_usetex=False,
-               font_family='monospace', font_weight='normal', font_size=12,
-               style='classic'):
-    plt.style.use(style)
-    plt.rcParams.update({'text.usetex': text_usetex})
-    plt.rcParams.update({'font.family': font_family})
-    plt.rcParams.update({'font.weight': font_weight})
-    plt.rcParams.update({'font.size': int(font_size)})
-
-
 def tick_formatter_short(x, p):
     X = str(x)
     if len(X) > 4:
         return '{:2g}'.format(x)
-    else:
-        return x
-
-
-@decorate_ax_style
-def ax_add_args_histo(label, color='blue', edgecolor='none'):
-    if not edgecolor:
-        edgecolor = color
-
-    return {
-        'color': color,
-        'edgecolor': 'none',
-        'label': label
-    }
-
-
-ax_add_args_simple = ax_add_args_histo  # For backward compatibility
-
-
-@decorate_ax_style
-def ax_add_args_errorbar(label, color, yerr=None, marker='o',
-                         markeredgecolor='none', ls='none'):
-    return {
-        'label': label,
-        'ls': ls,
-        'color': color,
-        'marker': marker,
-        'markeredgecolor': markeredgecolor,
-        'yerr': yerr
-    }
-
-
-@decorate_ax_style
-def ax_add_args_step(label, color, where='mid'):
-    return {
-        'label': label,
-        'color': color,
-        'where': where
-    }
-
-
-def ax_add_args_default(num, mean, std, *args, **kwargs):
-    return ax_add_args_simple(
-        'tot: {:.4g} mean: {:.2g} std: {:.2g}'.format(
-            num, mean, std
-        ),
-        *args, **kwargs)
+    return x
 
 
 def plot_prepare(figure=None, axis=None, title=None,
                  xtick_formatter=None, ytick_formatter=None,
                  xlabel=None, ylabel=None,
-                 xscale='linear', yscale='linear'):
+                 xlim=None, ylim=None,
+                 xscale='linear', yscale='linear',
+                 show_legend=True,
+                 legend_add_args={'numpoints': 1, 'loc': 'best'}):
     fig = plt.figure() if not figure else figure
 
     if not axis:
@@ -155,11 +102,17 @@ def plot_prepare(figure=None, axis=None, title=None,
     if ylabel:
         ax.set_ylabel(ylabel)
 
-    return fig, ax
+    if xlim:
+        ax.set_xlim(xlim)
+    if ylim:
+        ax.set_ylim(ylim)
 
+    if show_legend:
+        legend = lambda: ax.legend(**legend_add_args)
+    else:
+        legend = lambda: True
 
-def filter_kwargs_plot_prepare(kwargs):
-    return filter_kwargs_func(kwargs, plot_prepare)
+    return fig, ax, legend
 
 
 def get_ytick_position_in_ax_coordinate(ax, scale):
@@ -208,6 +161,54 @@ def convert_bins_to_central_pos(bins):
     return bins[:-1] + (np.diff(bins)/2)
 
 
+######################
+# Plot style helpers #
+######################
+
+@decorate_ax_style
+def ax_add_args_histo(label, color='blue', edgecolor='none'):
+    if not edgecolor:
+        edgecolor = color
+
+    return {
+        'color': color,
+        'edgecolor': 'none',
+        'label': label
+    }
+
+
+@decorate_ax_style
+def ax_add_args_errorbar(label, color, yerr=None, marker='o',
+                         markeredgecolor='none', ls='none'):
+    return {
+        'label': label,
+        'ls': ls,
+        'color': color,
+        'marker': marker,
+        'markeredgecolor': markeredgecolor,
+        'yerr': yerr
+    }
+
+
+@decorate_ax_style
+def ax_add_args_step(label, color, where='post'):
+    return {
+        'label': label,
+        'color': color,
+        'where': where
+    }
+
+
+@decorate_ax_style
+def ax_add_args_fill(color, alpha=0.5, linewidth=0., step='post'):
+    return {
+        'color': color,
+        'alpha': alpha,
+        'linewidth': linewidth,
+        'step': step
+    }
+
+
 ################
 # Simple plots #
 ################
@@ -216,7 +217,7 @@ def convert_bins_to_central_pos(bins):
 def plot_hexbin(x, y, gridsize, hexbin_add_args,
                 output=None, cmap='inferno', bins='log', colorbar_label=None,
                 **kwargs):
-    fig, ax = plot_prepare(**kwargs)
+    fig, ax, legend = plot_prepare(**kwargs)
     hb = ax.hexbin(x, y,
                    gridsize=gridsize, cmap=cmap, bins=bins, **hexbin_add_args)
     cb = fig.colorbar(hb, ax=ax)
@@ -224,133 +225,88 @@ def plot_hexbin(x, y, gridsize, hexbin_add_args,
     if colorbar_label:
         cb.set_label(colorbar_label)
 
+    legend()
     return output, fig, ax
 
 
 @decorate_output
-def plot_histo(histo, bins, histo_add_args,
+def plot_histo(bins, histo, histo_add_args,
                output=None, xtick_formatter=tick_formatter_short,
-               show_legend=True,
-               xlim=None, ylim=None,
                **kwargs):
-    fig, ax = plot_prepare(xtick_formatter=xtick_formatter, **kwargs)
-    ax.bar(bins[:-1], histo, width=np.diff(bins), align='edge',
-           **histo_add_args)
-
-    if show_legend:
-        ax.legend()
-
-    if xlim:
-        ax.set_xlim(xlim)
-
-    if ylim:
-        ax.set_ylim(ylim)
+    fig, ax, legend = plot_prepare(xtick_formatter=xtick_formatter, **kwargs)
+    mid_bins = convert_bins_to_central_pos(bins)
+    ax.bar(mid_bins, histo, width=np.diff(bins), **histo_add_args)
 
+    legend()
     return output, fig, ax
 
 
 @decorate_output
 def plot_errorbar(x, y, errorbar_add_args,
-                  output=None,
-                  convert_x=True,
-                  show_legend=True, legend_loc='best',
-                  xlim=None, ylim=None,
+                  output=None, convert_x=True,
                   **kwargs):
-    fig, ax = plot_prepare(**kwargs)
-
     if convert_x:
         x = convert_bins_to_central_pos(x)
 
+    fig, ax, legend = plot_prepare(**kwargs)
     ax.errorbar(x, y, **errorbar_add_args)
 
-    if show_legend:
-        ax.legend(numpoints=1, loc=legend_loc)
-
-    if xlim:
-        ax.set_xlim(xlim)
-
-    if ylim:
-        ax.set_ylim(ylim)
-
+    legend()
     return output, fig, ax
 
 
 @decorate_output
 def plot_step(x, y, step_add_args,
               output=None,
-              convert_x=True,
-              show_legend=True, legend_loc='best',
-              xlim=None, ylim=None,
               **kwargs):
-    fig, ax = plot_prepare(**kwargs)
-
-    if convert_x:
-        x = convert_bins_to_central_pos(x)
+    y = np.append(y, y[-1])
 
+    fig, ax, legend = plot_prepare(**kwargs)
     ax.step(x, y, **step_add_args)
 
-    if show_legend:
-        ax.legend(numpoints=1, loc=legend_loc)
-
-    if xlim:
-        ax.set_xlim(xlim)
-
-    if ylim:
-        ax.set_ylim(ylim)
-
+    legend()
     return output, fig, ax
 
 
-##########################
-# Common composite plots #
-##########################
+@decorate_output
+def plot_fill(x, y_range, fill_add_args,
+              output=None,
+              **kwargs):
+    ymin, ymax = y_range
+    ymin = np.append(ymin, ymin[-1])
+    ymax = np.append(ymax, ymax[-1])
+
+    fig, ax, legend = plot_prepare(**kwargs)
+    ax.fill_between(x, ymin, ymax, **fill_add_args)
+
+    legend()
+    return output, fig, ax
+
+
+###################
+# Composite plots #
+###################
 
 @decorate_output
-def plot_two_histos(histo1, bins1, histo2, bins2,
-                    histo1_add_args, histo2_add_args,
-                    output=None,
-                    **kwargs):
-    kw_plot_prepare, kw_rest = filter_kwargs_plot_prepare(kwargs)
-    fig, ax = plot_prepare(**kw_plot_prepare)
+def plot_top(top_plotters,
+             output=None,
+             **kwargs):
+    fig, ax, legend = plot_prepare(**kwargs)
 
-    _, ax1 = plot_histo(histo1, bins1, histo1_add_args, figure=fig, axis=ax,
-                        show_legend=False, **kw_rest)
-    _, ax2 = plot_histo(histo2, bins2, histo2_add_args, figure=fig, axis=ax1,
-                        **kw_rest)
+    for p in top_plotters:
+        p(fig, ax)
 
-    return output, fig, ax1, ax2
+    legend()
+    return output, fig, ax
 
 
-@decorate_output
-def plot_two_errorbar(x1, y1, x2, y2, errorbar1_add_args, errorbar2_add_args,
-                      output=None,
-                      **kwargs):
-    kw_plot_prepare, kw_rest = filter_kwargs_plot_prepare(kwargs)
-    fig, ax = plot_prepare(**kw_plot_prepare)
-
-    _, ax1 = plot_errorbar(x1, y1, errorbar1_add_args, figure=fig, axis=ax,
-                           show_legend=False, **kw_rest)
-    _, ax2 = plot_errorbar(x2, y2, errorbar2_add_args, figure=fig, axis=ax1,
-                           **kw_rest)
-
-    return output, fig, ax1, ax2
-
-
-##############
-# Grid plots #
-##############
-
-def plot_top_histo_bot_errorbar(histo1, bins1, histo2, bins2, x_ratio, y_ratio,
-                                histo1_add_args, histo2_add_args,
-                                output,
-                                ratio_add_args=ax_add_args_errorbar(
-                                    'Ratio', 'black'),
-                                title=None,
-                                xlabel=None,
-                                ax1_ylabel=None, ax2_ylabel=None,
-                                ax1_yscale='linear', ax2_yscale='linear',
-                                height_ratios=[3, 1],
-                                **kwargs):
+def plot_top_bot(top_plotters, bot_plotters,
+                 title=None,
+                 xlabel=None, ax1_ylabel=None, ax2_ylabel=None,
+                 ax1_yscale='linear', ax2_yscale='linear',
+                 height_ratios=[3, 1],
+                 legend_add_args={'numpoints': 1, 'loc': 'best'},
+                 **kwargs):
     fig = plt.figure()
     fig.set_tight_layout({'pad': 0.})
 
@@ -359,17 +315,28 @@ def plot_top_histo_bot_errorbar(histo1, bins1, histo2, bins2, x_ratio, y_ratio,
 
     ax1 = fig.add_subplot(spec[0, 0])
     ax1.set_yscale(ax1_yscale)
-    plot_two_histos(histo1, bins1, histo2, bins2,
-                    histo1_add_args, histo2_add_args,
-                    figure=fig, axis=ax1,
-                    ylabel=ax1_ylabel, title=title,
-                    **kwargs)
+
+    if ax1_ylabel:
+        ax1.set_ylabel(ax1_ylabel)
+    if title:
+        ax1.set_title(title)
 
     ax2 = fig.add_subplot(spec[1, 0], sharex=ax1)
     ax2.set_yscale(ax2_yscale)
-    plot_errorbar(x_ratio, y_ratio, ratio_add_args,
-                  figure=fig, axis=ax2,
-                  xlabel=xlabel, ylabel=ax2_ylabel, show_legend=False)
+
+    if xlabel:
+        ax2.set_xlabel(xlabel)
+    if ax2_ylabel:
+        ax2.set_ylabel(ax2_ylabel)
+
+    for p in top_plotters:
+        p(fig, ax1, **kwargs)
+
+    for p in bot_plotters:
+        p(fig, ax2, **kwargs)
+
+    # Always show legend for top
+    ax1.legend(**legend_add_args)
 
     # Remove the horizontal labels for the top plot
     for tick in ax1.xaxis.get_major_ticks():
@@ -385,56 +352,4 @@ def plot_top_histo_bot_errorbar(histo1, bins1, histo2, bins2, x_ratio, y_ratio,
     # Align y labels
     fig.align_ylabels()
 
-    fig.savefig(output)
-
-
-def plot_top_errorbar_bot_errorbar(x1, y1, x2, y2, x_ratio, y_ratio,
-                                   errorbar1_add_args, errorbar2_add_args,
-                                   output,
-                                   ratio_add_args=ax_add_args_errorbar(
-                                       'Ratio', 'black'),
-                                   title=None,
-                                   xlabel=None,
-                                   ax1_ylabel=None, ax2_ylabel=None,
-                                   ax1_yscale='linear', ax2_yscale='linear',
-                                   hline_pos=None,
-                                   height_ratios=[3, 1],
-                                   **kwargs):
-    fig = plt.figure()
-    fig.set_tight_layout({'pad': 0.})  # Remove surrounding padding
-
-    spec = fig.add_gridspec(ncols=1, nrows=2, height_ratios=height_ratios)
-    spec.update(hspace=0.)  # Remove gaps between subplots
-
-    ax1 = fig.add_subplot(spec[0, 0])
-    ax1.set_yscale(ax1_yscale)
-    plot_two_errorbar(x1, y1, x2, y2, errorbar1_add_args, errorbar2_add_args,
-                      figure=fig, axis=ax1,
-                      ylabel=ax1_ylabel, title=title, **kwargs)
-
-    ax2 = fig.add_subplot(spec[1, 0], sharex=ax1)
-    ax2.set_yscale(ax2_yscale)
-    plot_errorbar(x_ratio, y_ratio, ratio_add_args,
-                  figure=fig, axis=ax2,
-                  xlabel=xlabel, ylabel=ax2_ylabel, show_legend=False)
-
-    # Add a horizontal line
-    if not hline_pos:
-        hline_pos = y_ratio[y_ratio != 0].mean()
-    ax2.axhline(hline_pos, color='gray')
-
-    # Remove the horizontal labels for the top plot
-    for tick in ax1.xaxis.get_major_ticks():
-        tick.label1.set_visible(False)
-
-    # No offset (like +1 on top of the axis)
-    if ax2_yscale == 'linear':
-        ax2.ticklabel_format(axis='y', useOffset=False)
-
-    # Add some padding if the TOPMOST ylabel of bot is too close to the top
-    ensure_no_majortick_on_topmost(ax2, ax2_yscale)
-
-    # Align y labels
-    fig.align_ylabels()
-
-    fig.savefig(output)
+    return fig, ax1, ax2
